//   
// VECT_TAB_SRAM    RAM!

#include "dynamic_display.h"

//   -  .
GPIO_TypeDef* const PORTS[]={GPIOA, GPIOB,
			GPIOC, GPIOD};

//    -, 
//  (   XXXX_CONTROL)   state.
void line_set(char port_pin, bool state)
{
	if(state)
		PORTS[(port_pin>>4)&0xF]->BSRR=1<<(port_pin&0xF);
	else
		PORTS[(port_pin>>4)&0xF]->BRR=1<<(port_pin&0xF);

}

//    .
void digs_off()
{
	for(uint32_t i=0; i<DIGS_COUNT; i++)
		line_set(DIGS_CONTROL[i], !DIG_ACTIVE_LEVEL);
}

//   d,    0;
// d=0    .
void dig_on(uint32_t d)
{
	line_set(DIGS_CONTROL[d], DIG_ACTIVE_LEVEL);
}

//       b;
//     A,  - B
//  ..,      DP.
//    ,  .
void set_seg_bus(char b)
{
	for(uint32_t i=0; i<8; i++, b>>=1)
		line_set(SEGS_CONTROL[i], (b&1)^!SEG_ACTIVE_LEVEL);
}

//   ,
//  .
void pin_config(char pp, bool level)
{
	uint32_t port=(pp>>4)&0xF;
	uint32_t pin=pp&0xF;
	//    
	// (,    
	//  ).
	RCC->APB2ENR|=RCC_APB2ENR_IOPAEN<<port;
	if(level)
		PORTS[port]->BSRR=1<<pin;
	else
		PORTS[port]->BRR=1<<pin;
	uint32_t mask, fnmode;
	volatile uint32_t *mode_reg=(pin<8)?&PORTS[port]->CRL:
			&PORTS[port]->CRH;
	if(pin>=8)
		pin-=8;
	mask=~(0xF<<(4*pin));
	fnmode=2<<(4*pin);
	*mode_reg&=mask;
	*mode_reg|=fnmode;
}

//   , 
//   .
void display_config()
{
	for(uint32_t i=0; i<DIGS_COUNT; i++)
		pin_config(DIGS_CONTROL[i], !DIG_ACTIVE_LEVEL);
	for(uint32_t i=0; i<8; i++)
			pin_config(SEGS_CONTROL[i], !SEG_ACTIVE_LEVEL);
}

//      .
//    ASCII ,  
//    
// ( 0 -  '0'  ..).
//       
//       .
//        .
char OUT_STR[]={SPACE, SPACE, SPACE, SPACE};

const uint32_t OUT_STR_LENGTH=sizeof OUT_STR/sizeof OUT_STR[0];

//      ;
//      f=N*F,
//  N -   ,
// F -    .
void display_refresh()
{
	static uint32_t d=0;
	char ch=OUT_STR[d];
	char dot=ch&0x80;
	ch&=0x7F;
	ch=CG_TABLE[ch]|dot;
	digs_off();
	set_seg_bus(ch);
	dig_on(d);
	if(++d>=OUT_STR_LENGTH)
		d=0;
}

//      .
//     ,
//    : 'E'  '-E'
//    false.
bool display(int n)
{
	char sign=SPACE;
	if(n<0)
	{
		n=-n;
		sign=MINUS;
	}
	char d=0;
	for(uint32_t i=0; i<OUT_STR_LENGTH; i++)
	{
		if(n!=0)
		{
			d=n%10;
			n/=10;
		}
		if(i==0)
			d|=0x80; //    .
		else if(n==0&&d==0)
		{
			//  ,  ;
			//   .
			d=sign;
			sign=SPACE;
		}
		OUT_STR[OUT_STR_LENGTH-1-i]=d;
		d=0;
	}
	if(n!=0||sign!=SPACE)
	{
		for(uint32_t i=0; i<OUT_STR_LENGTH-1; i++)
			OUT_STR[i]=SPACE;
		OUT_STR[OUT_STR_LENGTH-1]=0xE;
		if(sign==MINUS&&OUT_STR_LENGTH>=2)
			OUT_STR[OUT_STR_LENGTH-2]=MINUS;
		return false;
	}
	return true;
}

//      n/(10**p).
//  true     false   .
bool display(int n, int p)
{
	bool fneg=n<0;
	bool ferr=false;
	if(fneg)
		n=-n;
	if(p>=(int)OUT_STR_LENGTH)
		ferr=true;
	else
	{
		for(int i=0; i<OUT_STR_LENGTH; i++)
		{
			char c=SPACE;
			if(i<=p||n!=0||i==0)
				c=n%10;
			else if(fneg)
			{
				fneg=false;
				c=MINUS;
			}
			if(i==p)
				c|=0x80;
			OUT_STR[OUT_STR_LENGTH-1-i]=c;
			n/=10;
		}
	}
	if(ferr||fneg||n!=0)
	{
		if(fneg&&n==0&&OUT_STR[0]==0x80)
		{
			OUT_STR[0]=0x80|MINUS;
			return true;
		}
		for(unsigned int i=0; i<OUT_STR_LENGTH; i++)
			OUT_STR[i]=SPACE;
		OUT_STR[OUT_STR_LENGTH-1]=0x0E;
		if(fneg)
			OUT_STR[OUT_STR_LENGTH-2]=MINUS;
		return false;
	}
	return true;
}
